Term

Enum Term 

pub enum Term {
    Id(Id),
    Selector(Selector),
}
Term.

Terms can either be identifiers or selectors, both of which are convertible into Selector. By providing Id, the term represents an exact match on identifiers, whereas providing a Selector allows for more complex matching criteria.

Variants



Id(Id)

Identifier.



Selector(Selector)

Selector.