Format

Struct Format 

pub struct Format<const N: usize> { /* private fields */ }
Formatted string.

This is a low-level construct which allows to efficiently work with strings that contain a predefined number of components separated by : characters. If a component contains a : itself, it is percent-encoded, indicated by a flag. This is slower, but not expected to be common.

Formatted strings are optimized for very fast conversion with FromStr or gradual construction through Format::builder, which both produce an immutable instance. To make sure cloning is fast, it’s recommended to wrap the encapsulating type, Selector or Id, in an Arc.

This implementation is currently limited to 64 spans, which should probably be sufficient for all use cases that can ever happen. For our means, an u8 would be more than enough, but since Rust will align the field to 64 bits anyway, there’s no point in being cheap.

Examples

use zrx_id::format::Format;

// Create formatted string builder
let mut builder = Format::<3>::builder();
builder.set(0, "a");
builder.set(1, "b");
builder.set(2, "c");

// Create formatted string from builder
let format = builder.build()?;

// Obtain string representation
assert_eq!(format.as_str(), "a:b:c");

Implementations

Source

impl<const N: usize> Format<N>

pub fn builder<'a>() -> Builder<'a, N>

Creates a formatted string builder.

Examples
use zrx_id::format::Format;

// Create formatted string builder
let mut builder = Format::<3>::builder();
pub fn to_builder(&self) -> Builder<'_, N>

Creates a formatted string builder from the formatted string.

This method creates a builder from the current formatted string, which allows to modify components and build a new formatted string.

Examples
use zrx_id::format::Format;

// Create formatted string from string
let format: Format::<3> = "a:b:c".parse()?;

// Create formatted string builder
let mut builder = format.to_builder();
builder.set(2, "d");

// Create formatted string from builder
let format = builder.build()?;
assert_eq!(format.as_str(), "a:b:d");
Source

impl<const N: usize> Format<N>

pub fn get(&self, index: usize) -> Cow<'_, str>

Returns the value at the index.

If the value is not percent-encoded, which means it does not contain a : character, a borrowed reference is returned which is essentially a zero-cost operation and expected to be the common case. Otherwise, the value is percent-decoded and an owned value is returned.

Panics

Panics if the index is out of bounds. Since Format is a low-level construct, we don’t expect this to happen, as indices should be known.

Examples
use zrx_id::format::Format;

// Create formatted string builder
let mut builder = Format::<3>::builder();
builder.set(0, "a");
builder.set(1, "b");
builder.set(2, "c");

// Create formatted string from builder
let format = builder.build()?;

// Obtain value at index
assert_eq!(format.get(0), "a");
pub fn as_str(&self) -> &str

Returns the string representation.

Examples
use zrx_id::format::Format;

// Create formatted string builder
let mut builder = Format::<3>::builder();
builder.set(0, "a");
builder.set(1, "b");
builder.set(2, "c");

// Create formatted string from builder
let format = builder.build()?;

// Obtain string representation
assert_eq!(format.as_str(), "a:b:c");