Expression

Struct Expression 

pub struct Expression { /* private fields */ }
Expression.

Expressions allow to build trees of Id and Selector instances combined with logical operators, enabling complex matching and filtering.

The following operators are supported:

Examples

use zrx_id::{selector, Expression};

// Create expression
let expr = Expression::all(|expr| {
    expr.with(selector!(location = "**/*.md")?)?
        .with(Expression::not(|expr| {
            expr.with(selector!(provider = "file")?)
        })
    )
})?;

Implementations

Source

impl Expression

pub fn any<F>(f: F) -> Result<Self>
where F: FnOnce(Builder) -> Result<Builder>,

Creates an expression for which any operand must match.

Errors

Returns Error::Id if any of the operands are invalid.

Examples
use zrx_id::{selector, Expression};

// Create expression
let expr = Expression::any(|expr| {
    expr.with(selector!(location = "**/*.jpg")?)?
        .with(selector!(location = "**/*.png")?)
})?;
pub fn all<F>(f: F) -> Result<Self>
where F: FnOnce(Builder) -> Result<Builder>,

Creates an expression for which all operands must match.

Errors

Returns Error::Id if any of the operands are invalid.

Examples
use zrx_id::{selector, Expression};

// Create expression
let expr = Expression::all(|expr| {
    expr.with(selector!(location = "**/*.md")?)?
        .with(selector!(provider = "file")?)
})?;
pub fn not<F>(f: F) -> Result<Self>
where F: FnOnce(Builder) -> Result<Builder>,

Creates an expression for which no operand must match.

Errors

Returns Error::Id if any of the operands are invalid.

Examples
use zrx_id::{selector, Expression};

// Create expression
let expr = Expression::all(|expr| {
    expr.with(selector!(location = "**/*.md")?)?
        .with(Expression::not(|expr| {
            expr.with(selector!(provider = "file")?)
        })
    )
})?;
Source

impl Expression

pub fn operator(&self) -> Operator

Returns the operator.

pub fn operands(&self) -> &[Operand]

Returns a reference to the operands.

pub fn len(&self) -> usize

Returns the number of operands.

pub fn is_empty(&self) -> bool

Returns whether there are any operands.