Id

Struct Id 

pub struct Id { /* private fields */ }
Identifier.

Identifiers are structured string-based representations which are used to uniquely identify artifacts as they move through streams and stores. They use a compact, yet human-readable format that is easy to generate and parse, and consists of the following six components:

  • provider, e.g., file or git.
  • resource, e.g., volume, branch or tag.
  • variant, e.g., language, version or format.
  • context, e.g., source or output directory.
  • location, e.g., file or folder.
  • fragment, e.g., line number or anchor.

Identifiers implement Eq, PartialEq and Hash, as well as Ord and PartialOrd, so they can be stored in ordered and unordered storages, as well as efficiently compared with each other. The structured string-based representation is defined as follows:

zri:<provider>:<resource>:<variant>:<context>:<location>:<fragment>

This ensures blazing fast cloning and editing. Additionally, identifiers are guaranteed to not contain backslashes or path traversals in components. An empty component, for those that are allowed to remain empty, is equal to the default in the context set by the given provider.

Examples

Create an identifier:

use zrx_id::Id;

// Create identifier builder
let builder = Id::builder()
    .provider("file")
    .context("docs")
    .location("index.md");

// Create identifier from builder
let id = builder.build()?;
assert_eq!(id.as_str(), "zri:file:::docs:index.md:");

Create an identifier from a string:

use zrx_id::Id;

// Create identifier from string
let id: Id = "zri:file:::docs:index.md:".parse()?;

Implementations

Source

impl Id

pub fn builder<'a>() -> Builder<'a>

Creates an identifier builder.

Examples
use zrx_id::Id;

// Create identifier builder
let builder = Id::builder();
pub fn to_builder(&self) -> Builder<'_>

Creates a builder from the identifier.

This method creates a builder from the current identifier, which allows to modify components and build a new identifier from an existing one.

Examples
use zrx_id::Id;

// Create identifier from string
let id: Id = "zri:file:::docs:index.md:".parse()?;

// Create identifier builder
let builder = id.to_builder().location("README.md");

// Create identifier from builder
let id = builder.build()?;
assert_eq!(id.as_str(), "zri:file:::docs:README.md:");
Source

impl Id

pub fn to_path(&self) -> PathBuf

Converts the identifier to a relative file system path.

This method creates a relative PathBuf from both, the context and location components of the identifier, using platform-dependent path separators. The resulting path is always relative, and never absolute, since both, context and location, are always relative.

In order to resolve the path, the Id::resource needs to be taken into account, which is of course provider-specific. Note that for use of paths in URLs, Id::as_uri must be used, which guarantees that all path separators are forward slashes.

Examples
use std::path::Path;
use zrx_id::Id;

// Create identifier from string
let id: Id = "zri:file:::docs:index.md:".parse()?;

// Create path from identifier
let path = id.to_path();
assert_eq!(path, Path::new("docs/index.md"));
pub fn as_str(&self) -> &str

Returns the string representation.

Examples
use zrx_id::Id;

// Create identifier from string
let id: Id = "zri:file:::docs:index.md:".parse()?;

// Obtain string representation
assert_eq!(id.as_str(), "zri:file:::docs:index.md:");
pub fn as_uri(&self) -> Uri<'_>

Returns the URI representation.

This method creates a URI from Id::location, which is necessary for using the identifier in URLs, e.g., to construct relative links.

Examples
use zrx_id::uri::Uri;
use zrx_id::Id;

// Create identifier from string
let id: Id = "zri:file:::docs:index.md:".parse()?;

// Obtain URI representation
assert_eq!(id.as_uri(), Uri::from("index.md"));
Source

impl Id

pub fn provider(&self) -> Cow<'_, str>

Returns the provider component.

pub fn resource(&self) -> Option<Cow<'_, str>>

Returns the resource component, if any.

pub fn variant(&self) -> Option<Cow<'_, str>>

Returns the variant component, if any.

pub fn context(&self) -> Cow<'_, str>

Returns the context component.

pub fn location(&self) -> Cow<'_, str>

Returns the location component.

pub fn fragment(&self) -> Option<Cow<'_, str>>

Returns the fragment component, if any.