Selector

Struct Selector 

pub struct Selector { /* private fields */ }
Selector.

Selectors are used to match identifiers. Like identifiers, they consist of six components, which can be set to specific values or left empty to act as wildcards. Each component can be set to a glob as supported by globset, which allows for powerful matching capabilities.

Selectors are no means to an end, but rather a building block to associate data or functions to identifiers via the construction of a Matcher, which uses an efficient algorithm to match an arbitrary set of selectors in linear time. While it’s recommended to use Selector::builder together with the associated methods to create a new selector, selectors can also be created from a structured string representation with Selector::from_str, which is used internally for serializing them to persistent storage:

zrs:<provider>:<resource>:<variant>:<context>:<location>:<fragment>

This ensures blazing fast cloning and editing. Additionally, selectors are guaranteed to not contain backslashes or path traversals in components. An empty component is interpreted as a wildcard, and thus matches all values in that component for any given selector.

Examples

Create a selector:

use zrx_id::Selector;

// Create selector builder
let builder = Selector::builder().location("**/*.md");

// Create selector from builder
let selector = builder.build()?;
assert_eq!(selector.as_str(), "zrs:::::**/*.md:");

Create a selector from a string:

use zrx_id::Selector;

// Create selector from string
let selector: Selector = "zrs:::::**/*.md:".parse()?;

Implementations

Source

impl Selector

pub fn builder<'a>() -> Builder<'a>

Creates a selector builder.

Examples
use zrx_id::Selector;

// Create selector builder
let builder = Selector::builder();
pub fn to_builder(&self) -> Builder<'_>

Creates a builder from the selector.

This method creates a builder from the current selector, which allows to modify components and build a new selector from an existing one.

Examples
use zrx_id::Selector;

// Create selector from string
let selector: Selector = "zrs:::::**/*.md:".parse()?;

// Create selector builder
let builder = selector.to_builder().location("index.md");

// Create selector from builder
let selector = builder.build()?;
assert_eq!(selector.as_str(), "zrs:::::index.md:");
Source

impl Selector

pub fn as_str(&self) -> &str

Returns the string representation.

Examples
use zrx_id::Selector;

// Create selector from string
let selector: Selector = "zrs:::::**/*.md:".parse()?;

// Obtain string representation
assert_eq!(selector.as_str(), "zrs:::::**/*.md:");
Source

impl Selector

pub fn provider(&self) -> Option<Cow<'_, str>>

Returns the provider component, if any.

pub fn resource(&self) -> Option<Cow<'_, str>>

Returns the resource component, if any.

pub fn variant(&self) -> Option<Cow<'_, str>>

Returns the variant component, if any.

pub fn context(&self) -> Option<Cow<'_, str>>

Returns the context component, if any.

pub fn location(&self) -> Option<Cow<'_, str>>

Returns the location component, if any.

pub fn fragment(&self) -> Option<Cow<'_, str>>

Returns the fragment component, if any.