Sequence

Struct Sequence 

pub struct Sequence { /* private fields */ }
Sequence.

Sequences are ordered collections of expressions that can be used to match identifiers. Sequences can contain gaps, which allow to implement flexible matching of identifiers with varying numbers of elements, including but not limited to prefix and suffix matching.

Each Element of a Sequence can either be an Expression or a Gap. An Expression is a logical combination of one or multiple Id and Selector instances, while a Gap represents a wildcard that can match any number of elements, including zero.

The following convenience constructors are provided:

Examples

use zrx_id::{selector, Sequence};

// Create sequence
let sequence = Sequence::from([
    selector!(location = "zensical.toml")?,
    selector!(location = "**/*.md")?,
]);

Implementations

Source

impl Sequence

pub fn prefix<T>(sequence: T) -> Self
where T: Into<Self>,

Creates a sequence where the given elements are a prefix.

Examples
use zrx_id::{selector, Sequence};

// Create sequence for a prefix
let sequence = Sequence::prefix([
    selector!(location = "zensical.toml")?,
]);
pub fn suffix<T>(sequence: T) -> Self
where T: Into<Self>,

Creates a sequence where the given elements are a suffix.

Examples
use zrx_id::{selector, Sequence};

// Create sequence for a suffix
let sequence = Sequence::suffix([
    selector!(location = "**/*.md")?,
]);
pub fn iter(&self) -> Iter<'_, Element>

Creates an iterator over the sequence.

Examples
use zrx_id::{selector, Sequence};

// Create sequence
let sequence = Sequence::from([
    selector!(location = "zensical.toml")?,
    selector!(location = "**/*.md")?,
]);

// Create iterator over sequence
for element in sequence.iter() {
    println!("{element:?}");
}
Source

impl Sequence

pub fn len(&self) -> usize

Returns the number of elements.

pub fn is_empty(&self) -> bool

Returns whether there are any elements.