pub struct Specified<T> { /* private fields */ }Expand description
Specified value.
This data type is a thin wrapper around a value of type T, augmenting the
value with its computed Specificity, so it can be efficiently ordered.
Note that specifities are ordered from lowest to highest, with the least
specific value being the first in order. It would be natural to reverse
this order, but this would lead to more complex implementations for when
a specified value is wrapped in an Option.
Implementations
Sourceimpl<T> Specified<T>
impl<T> Specified<T>
Sourcepub fn into_inner(self) -> T
pub fn into_inner(self) -> T
Returns the inner value, consuming the specified value.
Examples
use zrx_id::selector;
use zrx_id::specificity::Specified;
// Create selector
let selector = selector!(location = "**/*.md")?;
// Create specified value from selector
let value = Specified::from(selector.clone());
assert_eq!(value.into_inner(), selector);Sourceimpl<T> Specified<T>
impl<T> Specified<T>
Sourcepub fn specificity(&self) -> Specificity
pub fn specificity(&self) -> Specificity
Returns the specificity.