pub struct Expression { /* private fields */ }Expand description
Expression.
Expressions allow to build trees of Id and Selector instances
combined with logical operators, enabling complex matching and filtering.
The following operators are supported:
Expression::any: LogicalOR- any operand must match.Expression::all: LogicalAND- all operands must match.Expression::not: LogicalNOT- no operand must match.
Examples
use zrx_id::{selector, Expression};
// Create expression
let expr = Expression::all(|expr| {
expr.with(selector!(location = "**/*.md")?)?
.with(Expression::not(|expr| {
expr.with(selector!(provider = "file")?)
})
)
})?;Implementations
Sourceimpl Expression
impl Expression
Sourcepub fn any<F>(f: F) -> Result<Self>
pub fn any<F>(f: F) -> Result<Self>
Creates an expression for which any operand must match.
Errors
Returns Error::Id if any of the operands are invalid.
Examples
use zrx_id::{selector, Expression};
// Create expression
let expr = Expression::any(|expr| {
expr.with(selector!(location = "**/*.jpg")?)?
.with(selector!(location = "**/*.png")?)
})?;Sourcepub fn all<F>(f: F) -> Result<Self>
pub fn all<F>(f: F) -> Result<Self>
Creates an expression for which all operands must match.
Errors
Returns Error::Id if any of the operands are invalid.
Examples
use zrx_id::{selector, Expression};
// Create expression
let expr = Expression::all(|expr| {
expr.with(selector!(location = "**/*.md")?)?
.with(selector!(provider = "file")?)
})?;Sourcepub fn not<F>(f: F) -> Result<Self>
pub fn not<F>(f: F) -> Result<Self>
Creates an expression for which no operand must match.
Errors
Returns Error::Id if any of the operands are invalid.
Examples
use zrx_id::{selector, Expression};
// Create expression
let expr = Expression::all(|expr| {
expr.with(selector!(location = "**/*.md")?)?
.with(Expression::not(|expr| {
expr.with(selector!(provider = "file")?)
})
)
})?;Sourceimpl Expression
impl Expression