pub struct Sequence { /* private fields */ }Expand description
Sequence.
Sequences are ordered collections of expressions that can be used to match identifiers. Sequences can contain gaps, which allow to implement flexible matching of identifiers with varying numbers of elements, including but not limited to prefix and suffix matching.
Each Element of a Sequence can either be an Expression or a
Gap. An Expression is a logical combination of one or multiple
Id and Selector instances, while a Gap represents a
wildcard that can match any number of elements, including zero.
The following convenience constructors are provided:
Sequence::prefix: Creates a sequence for prefix matching.Sequence::suffix: Creates a sequence for suffix matching.
Examples
use zrx_id::{selector, Sequence};
// Create sequence
let sequence = Sequence::from([
selector!(location = "zensical.toml")?,
selector!(location = "**/*.md")?,
]);Implementations
Sourceimpl Sequence
impl Sequence
Sourcepub fn prefix<T>(sequence: T) -> Selfwhere
T: Into<Self>,
pub fn prefix<T>(sequence: T) -> Selfwhere
T: Into<Self>,
Creates a sequence where the given elements are a prefix.
Examples
use zrx_id::{selector, Sequence};
// Create sequence for a prefix
let sequence = Sequence::prefix([
selector!(location = "zensical.toml")?,
]);Sourcepub fn suffix<T>(sequence: T) -> Selfwhere
T: Into<Self>,
pub fn suffix<T>(sequence: T) -> Selfwhere
T: Into<Self>,
Creates a sequence where the given elements are a suffix.
Examples
use zrx_id::{selector, Sequence};
// Create sequence for a suffix
let sequence = Sequence::suffix([
selector!(location = "**/*.md")?,
]);Sourcepub fn iter(&self) -> Iter<'_, Element>
pub fn iter(&self) -> Iter<'_, Element>
Creates an iterator over the sequence.
Examples
use zrx_id::{selector, Sequence};
// Create sequence
let sequence = Sequence::from([
selector!(location = "zensical.toml")?,
selector!(location = "**/*.md")?,
]);
// Create iterator over sequence
for element in sequence.iter() {
println!("{element:?}");
}Trait Implementations
impl Eq for Sequence
Sourceimpl<E> FromIterator<E> for Sequence
impl<E> FromIterator<E> for Sequence
Sourcefn from_iter<T>(iter: T) -> Selfwhere
T: IntoIterator<Item = E>,
fn from_iter<T>(iter: T) -> Selfwhere
T: IntoIterator<Item = E>,
Creates a sequence from an iterator.
Examples
use zrx_id::{selector, Sequence};
// Create sequence from iterator
let sequence = Sequence::from_iter([
selector!(location = "zensical.toml")?,
selector!(location = "**/*.md")?,
]);Sourceimpl<'a> IntoIterator for &'a Sequence
impl<'a> IntoIterator for &'a Sequence
Sourcefn into_iter(self) -> Self::IntoIter
fn into_iter(self) -> Self::IntoIter
Creates an iterator over the sequence.
Examples
use zrx_id::{selector, Sequence};
// Create sequence
let sequence = Sequence::from([
selector!(location = "zensical.toml")?,
selector!(location = "**/*.md")?,
]);
// Create iterator over sequence
for element in &sequence {
println!("{element:?}");
}Sourceimpl IntoIterator for Sequence
impl IntoIterator for Sequence
Sourcefn into_iter(self) -> Self::IntoIter
fn into_iter(self) -> Self::IntoIter
Creates a consuming iterator over the sequence.
Examples
use zrx_id::{selector, Sequence};
// Create sequence
let sequence = Sequence::from([
selector!(location = "zensical.toml")?,
selector!(location = "**/*.md")?,
]);
// Create iterator over sequence
for element in sequence {
println!("{element:?}");
}impl StructuralPartialEq for Sequence
Sourceimpl ToSpecificity for Sequence
impl ToSpecificity for Sequence
Sourcefn to_specificity(&self) -> Specificity
fn to_specificity(&self) -> Specificity
Computes the specificity of the sequence.
Examples
use zrx_id::selector;
use zrx_id::sequence::Sequence;
use zrx_id::specificity::ToSpecificity;
// Create sequence and compute specificity
let sequence = Sequence::from(selector!(location = "**/*.md")?);
assert_eq!(sequence.to_specificity(), (0, 1, 1, 3).into());impl Value for Sequence